#
#    Copyright (C) 2003-2004  James Tauber
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

"""
This module deals with session and user management.
"""


# session status codes
OK          = "ok"
WRONG_IP    = "different ip address"
UNKNOWN_SID = "unknown sid"
NO_SID      = "no sid in cookie"
NO_COOKIE   = "no cookie"



class SessionManager:


    def __init__(self, request, data_dir):
        self.request = request
        self.data_dir = data_dir
        self.session_db = None


    def check_session(self):
        """Returns the current session status as triple:
           code, user_id, session_id"""
        
        if self.request.cookie:
            import Cookie
            ip_address = self.request.remote_address
            cookie = Cookie.SimpleCookie(self.request.cookie)
            if "session_id" in cookie:
                sid = cookie["session_id"].value
                if self.has_session(sid):
                    session_uid, session_ip_address = self.get_session(sid)
                    if ip_address == session_ip_address:
                        return OK, session_uid, sid
                    else: # spoof
                        return WRONG_IP, None, sid
                else: # unknown session ID
                    return UNKNOWN_SID, None, sid
            else: # no session ID in cookie
                return NO_SID, None, None
        else: # no cookie
            return NO_COOKIE, None, None

        
    def get_sessions(self):
        """Get the session database, loading from disk if necessary"""
        
        if not self.session_db:
            import anydbm
            self.session_db = anydbm.open(self.data_dir + "sessions_db", "c")

        return self.session_db


    def create_session(self, uid, ip_address):
        """Create a new session for the given user at the given IP address"""
        
        sid  = generate_id(uid, ip_address)
        self.get_sessions()[sid] = " ".join((uid, ip_address))
        if hasattr(self.get_sessions(), "sync"):
            self.get_sessions().sync()
        return sid


    def get_session(self, sid):
        """Get the session info for the given session ID. Returned as pair:
           user_id, ip_address"""
        
        return self.get_sessions()[sid].split(" ")


    def has_session(self, sid):
        """Does the given session ID exist?"""
        
        return self.get_sessions().has_key(sid)


    def delete_session(self, sid):
        """Delete the session with the given ID. Has effect of logging user 
           out"""
        
        del self.get_sessions()[sid]
        if hasattr(self.get_sessions(), "sync"):
            self.get_sessions().sync()



def generate_id(uid, ip_address):
    """Generate a new session ID"""
    
    import md5, time, base64
    m = md5.new()
    m.update('this is just a fun little sentence to get things started')
    m.update(str(time.time()))
    m.update(uid)
    m.update(ip_address)
    return base64.encodestring(m.digest())[:-3].replace("/", "$")



class UserManager:


    def __init__(self, request, data_dir):
        self.request = request
        self.user_db = None
        self.data_dir = data_dir


    def get_users(self):
        """Get the user database, loading from disk if necessary"""
        
        if not self.user_db:
            import anydbm
            self.user_db = anydbm.open(self.data_dir + "user_db", "c")
            
        return self.user_db


    def known_user(self, user_id):
        """Is the given user known?"""
        
        return self.get_users().has_key(user_id)


    def get_password(self, user_id):
        """Get the (hashed) password for given user"""
        
        return self.get_userinfo(user_id)[2]


    def get_userinfo(self, user_id):
        """Get info for the given user. Returned as triple:
           user_id, name, (hashed) password"""
        
        return self.get_users()[user_id].split("|")


    def correct_password(self, user_id, passwd):
        """Is the given password correct for the given user?"""
        
        import sha
        return (self.known_user(user_id) and self.get_password(user_id) == sha.sha(passwd).hexdigest())
