#
#    Copyright (C) 2003-2005  James Tauber
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#


__provider__      = "DraftProvider"
__contributions__ = {"menu": "draft_menu_part"}    



import re
import time
import urllib

from formatters.manager import FormatterManager
from web_resource import PagePart

from providers.page.page_provider import HTMLPage, ForbiddenPage

import templates

def draft_menu_part(request, resource, main_resource, config):

    if request.logged_in():
        draft_href = config.get("draft", "draft_href")
        menu_part = templates.draft_menu % locals()
    else:
        menu_part = ""

    return menu_part


# form field names
page_id_name = "page_id"
page_title_name = "page_title"
page_content_name = "page_content"
page_content_type_name = "page_content_type"


# requires 'editform', 'blog' and 'put' modules

# @@@ perhaps this could delegate more to the editform provider

class DraftProvider:


    def __init__(self, resource_manager, lfs, config):

        self.resource_manager = resource_manager
        self.lfs              = lfs
        self.config           = config

        self.draft_prefix       = config.get("draft", "draft_prefix")
        
        self.formatter_manager = FormatterManager(config)


    def get(self, resource_id, request, main_resource):

        return self.handle(resource_id, request, main_resource)
        

    def post(self, resource_id, request, main_resource):
    
        return self.handle(resource_id, request, main_resource)
        

    def handle(self, resource_id, request, main_resource):
        """
        handle both GET and POST.
        """
    
        if not resource_id.startswith(self.draft_prefix):
            return None

        if not request.logged_in():
            return ForbiddenPage(resource_id, self.resource_manager, request, main_resource, self.config)

        draft_path = resource_id[len(self.draft_prefix):].strip("/")
        
        if draft_path == "": # top level
            page_part = DraftListPagePart(resource_id, self.resource_manager, request, self.lfs, self.config, self.formatter_manager)
            return HTMLPage(resource_id, page_part, self.resource_manager, request, main_resource, self.config)

        else: # individual draft page
            if main_resource != None:  # if not a top-level request,
                return None            # don't handle it here

            outer_page_part = DraftPagePart(resource_id, self.resource_manager, request, self.config)
            return HTMLPage(resource_id, outer_page_part, self.resource_manager, request, main_resource, self.config)



class DraftListPagePart(PagePart):


    def __init__(self, resource_id, resource_manager, request, lfs, config, formatter_manager):

        PagePart.__init__(self, resource_id)

        status = ""

        # @@@ how can this be avoided?
        page_content_name = globals()["page_content_name"]
        page_content_type_name = globals()["page_content_type_name"]
        page_title_name = globals()["page_title_name"]

        form_extras = ""
        for provider in resource_manager.get_contributors("edit_form"):
            form_extras += provider(request, None, None, config)
        
        form_content = templates.draft_initial_form_content % locals()

        # @@@ posting should really be done as a separate method
        if request.method == "POST":
            page_content = request.fields.getvalue(page_content_name, "")
            page_content_type = request.fields.getvalue(page_content_type_name, "")
            page_title = request.fields.getvalue(page_title_name, None)
                        
            if page_title:
                draft_resource_id = config.get("draft", "draft_prefix") + "/" + re.sub(' ', '_', page_title.lower())
                
                page = resource_manager.get(draft_resource_id, request, self)

                if page.exists():
                    status = templates.draft_existing_title % locals()
                    form_content = page_content
                else:
                    page.set_content(page_content, page_content_type)
                    page.set_property("page_title", page_title)

                    for provider in resource_manager.get_contributors("post_processing"):
                        provider(request, page, None, config)

                    status = templates.draft_success % locals()
            else:
                status = templates.draft_no_title % locals()
                form_content = page_content
        
        title = templates.draft_title
        
        # get list of drafts
        
        drafts = lfs.get_children(config.get("draft", "draft_prefix"))[1]
    
        if drafts:
            
            draft_items = ""
            for draft_title in drafts:
                draft_link = config.get("draft", "draft_href") + "/" + urllib.quote(draft_title)
                draft_items += templates.draft_item % locals()

            draft_list = templates.draft_list % locals()
        else:
            draft_list = templates.empty_draft_list % locals()

        action = config.get("draft", "draft_href")
        
        page_content_type = config.get("draft", "draft_type_default")
        
        content = templates.draft_page % locals()

        self.title = title
        self.html = content



class DraftPagePart(PagePart):


    def __init__(self, resource_id, resource_manager, request, config):
    
        PagePart.__init__(self, resource_id)
    
        page_part = resource_manager.get(resource_id, request, self)
        
        if page_part.exists():
            
            page_title = page_part.get_title()
            content = page_part.get_html()
            page_content = page_part.get_content()
            page_content_type = page_part.get_content_type()
    
            page_content = re.sub(r'&', '&amp;', page_content)
            page_content = re.sub(r'<', '&lt;', page_content)
            page_content = re.sub(r'>', '&gt;', page_content)
            page_content = re.sub(r'"', '&quot;', page_content)
    
            draft_path = resource_id[len(config.get("draft", "draft_prefix")):].strip("/")
    
            post_action = config.get("put", "put_href")
            resource_name = config.get("put", "put_resource_field")
            page_content_name = config.get("put", "put_content_field")
            page_content_type_name = config.get("put", "put_content_type_field")
            
            # @@@ should be in put config, not editform
            page_title_name = config.get("editform", "edit_title_field")
    
            resource_id_to_put = config.get("blog", "blog_path") + "/%04d/%02d/%02d/" % tuple(time.localtime()[:3]) + draft_path

            form_extras = ""
            for provider in resource_manager.get_contributors("draft_hidden"):
                form_extras += provider(request, page_part, None, config)
            
            content = templates.draft_put_form % locals() + content
    
            self.title = page_title
            self.html = content

        else: # @@@ really should return a 404
            self.title = "Non-existent draft" # @@@ templatize
            self.html = "<p>Non-existent draft.</p>" # @@@ templatize
        
    
    def is_read_only(self):
        return False
