#
#    Copyright (C) 2003-2005  James Tauber
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

"""
This module defines request and response objects for handling communication
from and to the client (e.g. web browser).
"""

import os
import cgi
import time

import auth
        
from auth import OK



class Request:


    def __init__(self, config):

        self.env            = os.environ

        self.fields         = cgi.FieldStorage()
        self.path_info      = self.env.get("PATH_INFO", "/")
        self.relative_path  = self.path_info.strip("/")
        self.method         = self.env.get("REQUEST_METHOD", "")
        self.query          = self.env.get("QUERY_STRING", "")
        self.remote_address = self.env.get("REMOTE_ADDR", "")
        self.cookie         = self.env.get("HTTP_COOKIE", None)
        
        self.if_modified_since = self.env.get("HTTP_IF_MODIFIED_SINCE", None)
        self.if_none_match     = self.env.get("HTTP_IF_NONE_MATCH", None)

        self.session_mgr = auth.SessionManager(self, config.get("DEFAULT", "data_dir"))
        self.user_mgr = auth.UserManager(self, config.get("DEFAULT", "data_dir"))

        
    def logged_in(self):
        """Is the user logged in?"""
    
        return (self.session_mgr.check_session()[0] == OK)


    def get_user(self):
        """Get the current user"""
    
        return self.session_mgr.check_session()[1]


    def correct_password(self, user_id, passwd):
        """Is the given user id, password combination correct?"""
        return self.user_mgr.correct_password(user_id, passwd)
        

    def check_session(self):
        return self.session_mgr.check_session()
        
        
    def create_session(self, uid, ip_address):
        return self.session_mgr.create_session(uid, ip_address)


    def delete_session(self, sid):
        return self.session_mgr.delete_session(sid)


    def not_modified(self, resource):
        if hasattr(resource, "lastmod"):
            if self.if_modified_since:
                lastmod = time.strptime(self.if_modified_since, "%a, %d %b %Y %H:%M:%S %Z")
                if time.mktime(resource.lastmod) <= time.mktime(lastmod):
                    return True
        return False
        
        

class Response:

    
    def __init__(self, resource=None, status=None):
    
        self.status = status
        self.cookie = None
        self.content = None
        self.content_type = None
        self.lastmod = None
        self.headers = {}

        if resource:    
            self.content_type = resource.get_mime_type()
            self.content = resource.get_content()
        
            if hasattr(resource, "status"):
                self.status = resource.status
            if hasattr(resource, "cookie"):
                self.cookie = resource.cookie
            if hasattr(resource, "expiry"):
                self.headers["Expires"] = 0
            if hasattr(resource, "lastmod"):
                lastmod = time.strftime("%a, %d %b %Y %H:%M:%S GMT", resource.lastmod)
                self.headers["Last-Modified"] = lastmod
                

    def send(self, out):
    
        if self.status:
            out.write("Status: %s\n" % self.status)
        for name, value in self.headers.items():
            out.write("%s: %s\n" % (name, value))
        if self.cookie:
            out.write("%s\n" % self.cookie)
        if self.content_type:
            out.write("Content-type: %s\n" % self.content_type)
        if self.content:
            out.write("\n")
            out.write(self.content)
